/**~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Licenciado bajo el esquema Academic Free License version 2.1 
 *
 * Proyecto Cupi2 (http://cupi2.uniandes.edu.co)
 * Ejercicio: n7_cupiCava
 * Autor: Equipo Cupi2 2015
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 */
package uniandes.cupi2.cupiCava.interfaz;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;

import uniandes.cupi2.cupiCava.mundo.ColorVino;
import uniandes.cupi2.cupiCava.mundo.PresentacionVino;
import uniandes.cupi2.cupiCava.mundo.TipoVino;
import uniandes.cupi2.cupiCava.mundo.Vino;

/**
 * Dilogo que permite agregar un nuevo vino a la cava.
 */
public class DialogoAgregarVino extends JDialog implements ActionListener
{
    // -----------------------------------------------------------------
    // Constantes
    // -----------------------------------------------------------------

    /**
     * Constante que representa el comando de agregar.
     */
    private final static String AGREGAR = "Agregar";

    /**
     * Constante que representa el comando para seleccionar una imagen.
     */
    private final static String SELECCIONAR = "Seleccionar";

    // -----------------------------------------------------------------
    // Atributos
    // -----------------------------------------------------------------

    /**
     * Ventana principal de la aplicacin
     */
    private InterfazCupiCava principal;

    // -----------------------------------------------------------------
    // Atributos de la Interfaz
    // -----------------------------------------------------------------

    /**
     * Campo de texto con el nombre del vino.
     */
    private JTextField txtNombre;

    /**
     * Combo box con las presentaciones del vino.
     */
    private JComboBox comboPresentacion;

    /**
     * Campo de texto con el ao de elaboracin del vino.
     */
    private JTextField txtAnhoElaboracion;

    /**
     * Campo de texto con el contenido en azcar del vino.
     */
    private JTextField txtContenidoAzucar;

    /**
     * Campo de texto con el lugar de origen del vino.
     */
    private JTextField txtLugarOrigen;

    /**
     * Campo de texto con la imagen del vino.
     */
    private JTextField txtImagen;

    /**
     * Combo box con los colores del vino.
     */
    private JComboBox comboColor;

    /**
     * Botn para agregar.
     */
    private JButton btnAgregar;

    /**
     * Botn para seleccionar.
     */
    private JButton btnSeleccionar;

    // -----------------------------------------------------------------
    // Constructor
    // -----------------------------------------------------------------

    /**
     * Constructor del dilogo.
     * @param pPrincipal Ventana principal de la aplicacin. pPrincipal != null.
     */
    public DialogoAgregarVino( InterfazCupiCava pPrincipal )
    {
        principal = pPrincipal;

        setLayout( new BorderLayout( ) );
        setSize( 450, 280 );
        setModal( true );
        setLocationRelativeTo( null );
        setTitle( "Agregar vino" );

        JPanel panelInfo1 = new JPanel( );
        panelInfo1.setLayout( new GridLayout( 8, 2 ) );
        panelInfo1.setBorder( new EmptyBorder( 5, 5, 5, 5 ) );

        panelInfo1.add( new JLabel( "Nombre:" ) );
        txtNombre = new JTextField( );
        panelInfo1.add( txtNombre );

        panelInfo1.add( new JLabel( "Presentacin:" ) );
        comboPresentacion = new JComboBox( );
        for(PresentacionVino t: PresentacionVino.values())
        {
        	comboPresentacion.addItem( t );
        }
        panelInfo1.add( comboPresentacion );

        panelInfo1.add( new JLabel( "Ao de elaboracin:" ) );
        txtAnhoElaboracion = new JTextField( );
        panelInfo1.add( txtAnhoElaboracion );

        panelInfo1.add( new JLabel( "Contenido en azcar (g/l):" ) );
        txtContenidoAzucar = new JTextField( );
        panelInfo1.add( txtContenidoAzucar );

        panelInfo1.add( new JLabel( "Lugar de origen:" ) );
        txtLugarOrigen = new JTextField( );
        panelInfo1.add( txtLugarOrigen );

        panelInfo1.add( new JLabel( "Color:" ) );
        comboColor = new JComboBox( );
        for (ColorVino c : ColorVino.values()) {
			comboColor.addItem(c);
		}
        panelInfo1.add( comboColor );

        add( panelInfo1, BorderLayout.CENTER );

        panelInfo1.add( new JLabel( "Imagen:" ) );

        JPanel panelAuxiliar = new JPanel( );
        panelAuxiliar.setLayout( new GridLayout( 1, 2 ) );
        txtImagen = new JTextField( );
        txtImagen.setEditable( false );
        btnSeleccionar = new JButton( "Seleccionar" );
        btnSeleccionar.addActionListener( this );
        btnSeleccionar.setActionCommand( SELECCIONAR );
        panelAuxiliar.add( txtImagen );
        panelAuxiliar.add( btnSeleccionar );

        panelInfo1.add( panelAuxiliar );

        btnAgregar = new JButton( AGREGAR );
        btnAgregar.setActionCommand( AGREGAR );
        btnAgregar.addActionListener( this );

        add( btnAgregar, BorderLayout.SOUTH );

    }

    // -----------------------------------------------------------------
    // Mtodos
    // -----------------------------------------------------------------

    /**
     * Manejo de los eventos de los botones.
     * @param pEvento Accin que gener el evento.
     */
    public void actionPerformed( ActionEvent pEvento )
    {
        String comando = pEvento.getActionCommand( );
        if( comando.equals( SELECCIONAR ) )
        {
            JFileChooser jf = new JFileChooser( "./data/imagenes" );
            jf.setDialogTitle( "Seleccionar archivo" );
            jf.setFileSelectionMode( JFileChooser.FILES_ONLY );
            jf.setVisible( true );

            if( jf.showOpenDialog( this ) == JFileChooser.APPROVE_OPTION )
            {
                String nombre = jf.getSelectedFile( ).getName( );
                if( nombre.endsWith( ".jpg" ) )
                {
                    txtImagen.setText( nombre );
                }
                else
                {
                    JOptionPane.showMessageDialog( this, "Debe seleccionar un archivo en formato .jpg.", "Seleccionar imagen", JOptionPane.ERROR_MESSAGE );
                }
            }
        }
        else if( comando.equals( AGREGAR ) )
        {
            String nombre = txtNombre.getText( );
            String anhoS = txtAnhoElaboracion.getText( );
            String azucarS = txtContenidoAzucar.getText( );
            String lugar = txtLugarOrigen.getText( );
            String imagen = txtImagen.getText( );

            PresentacionVino presentacion = ( PresentacionVino )comboPresentacion.getSelectedItem( );
            ColorVino color = ( ColorVino )comboColor.getSelectedItem( );

            if( nombre != null && !nombre.isEmpty( ) && anhoS != null && !anhoS.isEmpty( ) && azucarS != null && !azucarS.isEmpty( ) && lugar != null && !lugar.isEmpty( ) && imagen != null && !imagen.isEmpty( ) )
            {
                try
                {
                    int anho = Integer.parseInt( anhoS );
                    double azucar = Double.parseDouble( azucarS );
                    String imagen2 = "./data/imagenes/" + imagen;

                    if( anho > 0 )
                    {
                      principal.agregarVino( nombre, presentacion, anho, azucar, color, lugar, imagen2 );
                      this.dispose( );
                           
                    }
                    else
                    {
                        JOptionPane.showMessageDialog( this, "El ao de elaboracin debe ser mayor a cero.", "Agregar vino", JOptionPane.ERROR_MESSAGE );
                    }
                }
                catch( NumberFormatException ex )
                {
                    JOptionPane.showMessageDialog( this, "El ao de elaboracin y el contenido en azcar del vino deben ser valores numricos.", "Agregar vino", JOptionPane.ERROR_MESSAGE );
                }
            }
            else
            {
                JOptionPane.showMessageDialog( this, "Por favor ingrese la informacin completa del vino.", "Agregar vino", JOptionPane.ERROR_MESSAGE );
            }
        }
    }
}