/**~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Licenciado bajo el esquema Academic Free License version 2.1 
 *
 * Proyecto Cupi2 (http://cupi2.uniandes.edu.co)
 * Ejercicio: n7_cupiCava
 * Autor: Equipo Cupi2 2015
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 */
package uniandes.cupi2.cupiCava.interfaz;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.List;
import java.util.Properties;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import uniandes.cupi2.cupiCava.mundo.ColorVino;
import uniandes.cupi2.cupiCava.mundo.CupiCava;
import uniandes.cupi2.cupiCava.mundo.PresentacionVino;
import uniandes.cupi2.cupiCava.mundo.TipoVino;
import uniandes.cupi2.cupiCava.mundo.Vino;
import uniandes.cupi2.cupiCava.ordenamiento.Comparador;
import uniandes.cupi2.cupiCava.ordenamiento.ComparadorVinoNombre;
import uniandes.cupi2.cupiCava.ordenamiento.Ordenador;
import uniandes.cupi2.cupiCava.ordenamiento.Ordenamiento;

/**
 * Ventana principal de la aplicacin.
 */
public class InterfazCupiCava extends JFrame
{
    // -----------------------------------------------------------------
    // Constantes
    // -----------------------------------------------------------------

    /**
     * Constante que representa la ubicacin del archivo con los datos de los vinos de la cava.
     */
    private final static String RUTA_ARCHIVO = "./data/cava.properties";

    // -----------------------------------------------------------------
    // Atributos
    // -----------------------------------------------------------------

    /**
     * Clase principal del mundo.
     */
    private CupiCava cupiCava;
    
    /**
     * Clase que se encarga de hacer el ordenamiento de la lista de vinos
     */
    private Ordenador<Vino> ordenador;

    // -----------------------------------------------------------------
    // Atributos de la interfaz
    // -----------------------------------------------------------------

    /**
     * Panel con las extensiones.
     */
    private PanelExtension panelExtension;

    /**
     * Panel con la imagen del encabezado.
     */
    private PanelImagen panelImagen;

    /**
     * Panel con la lista de vinos de la cava.
     */
    private PanelListaVinos panelListaVinos;

    /**
     * Panel con la informacin detallada de un vino.
     */
    private PanelInformacionVino panelInformacionVino;

    /**
     * Panel con las opciones de ordenamiento y bsqueda.
     */
    private PanelOpciones panelOpciones;

    // -----------------------------------------------------------------
    // Constructores
    // -----------------------------------------------------------------

    /**
     * Constructor de la ventana principal.<br>
     * <b> post: </b> Construye la ventana principal de la aplicacin.
     */
    public InterfazCupiCava( )
    {
        cupiCava = new CupiCava( );
        cargarVinos( );
        ordenador = new Ordenador<Vino>();

        setLayout( new BorderLayout( ) );
        setSize( 700, 700 );
        setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );
        setTitle( "CupiCava" );
        setLocationRelativeTo( null );
        setResizable( false );

        panelImagen = new PanelImagen( );
        add( panelImagen, BorderLayout.NORTH );

        panelListaVinos = new PanelListaVinos( this );
        add( panelListaVinos, BorderLayout.WEST );

        panelInformacionVino = new PanelInformacionVino( );
        add( panelInformacionVino, BorderLayout.CENTER );

        JPanel aux = new JPanel( );
        aux.setLayout( new BorderLayout() );

        panelOpciones = new PanelOpciones( this );
        aux.add( panelOpciones, BorderLayout.CENTER );

        panelExtension = new PanelExtension( this );
        aux.add( panelExtension, BorderLayout.SOUTH );

        add( aux, BorderLayout.SOUTH );

        actualizarLista( );
    }

    // -----------------------------------------------------------------
    // Mtodos
    // -----------------------------------------------------------------

    /**
     * Carga los vinos iniciales de la cava a partir de un archivo de propiedades.
     */
    private void cargarVinos( )
    {
        try
        {
            FileInputStream fis = new FileInputStream( new File( RUTA_ARCHIVO ) );
            Properties propiedades = new Properties( );
            propiedades.load( fis );

            String aux = propiedades.getProperty( "total.vinos" );
            int numVinos = Integer.parseInt( aux );

            String dato;
            String nombre;
            String presentacion;
            int anho;
            double contenidoAzucar;
            String tipo;
            String lugar;
            String color;
            String imagen;

            for( int i = 1; i < numVinos + 1; i++ )
            {
                // Carga un vino
                dato = "vino" + i + ".nombre";
                nombre = propiedades.getProperty( dato );

                dato = "vino" + i + ".presentacion";
                presentacion = propiedades.getProperty( dato );

                dato = "vino" + i + ".fecha";
                anho = Integer.parseInt( propiedades.getProperty( dato ) );

                dato = "vino" + i + ".azucar";
                contenidoAzucar = Double.parseDouble( propiedades.getProperty( dato ) );

                dato = "vino" + i + ".tipo";
                tipo = propiedades.getProperty( dato );

                dato = "vino" + i + ".lugar";
                lugar = propiedades.getProperty( dato );

                dato = "vino" + i + ".color";
                color = propiedades.getProperty( dato );

                dato = "vino" + i + ".imagen";
                imagen = propiedades.getProperty( dato );

                if( nombre != null && !nombre.isEmpty( ) && presentacion != null && !presentacion.isEmpty( ) && anho > 0 && tipo != null && !tipo.isEmpty( ) && contenidoAzucar >= 0 && lugar != null && !lugar.isEmpty( ) && color != null
                        && !color.isEmpty( ) && imagen != null && !imagen.isEmpty( ) )
                {
                   cupiCava.agregarVino( nombre, PresentacionVino.valueOf(presentacion), anho, contenidoAzucar, ColorVino.valueOf(color), lugar, imagen );
                }

            }
            fis.close( );
        }
        catch( FileNotFoundException e )
        {
            e.printStackTrace( );
        }
        catch( IOException e )
        {
            e.printStackTrace( );
        }
    }

    /**
     * Actualiza la lista de vinos mostrada.
     */
    private void actualizarLista( )
    {
        panelListaVinos.refrescarLista( cupiCava.darVinos());
    }

    /**
     * Actualiza la informacin del vino seleccionado.
     * @param pNombreVino Nombre del vino seleccionado. pNombreVino != null && pNombreVino != "".
     */
    public void actualizarInfoVino( Vino seleccionado )
    {
        panelInformacionVino.actualizarInformacion( seleccionado.darNombre( ), seleccionado.darPresentacion( ), seleccionado.darAnhoElaboracion( ), seleccionado.darContenidoAzucar( ), seleccionado.darTipo( ), seleccionado.darLugarOrigen( ),
                seleccionado.darColor( ), seleccionado.darImagen( ) );
    }
    
    /**
     * Mtodo que permite ordenar la lista de vinos segn los criterios seleccionados por el usuario
     * @param ordenamiento algoritmo que se desea usar para ordenar los vinos. ordenamiento pertenece a Ordenamieto
     * @param comparador criterio que se desea usar para ordenar los vinos. comparador pertenece a Comaprador
     * @param ascendete indica si debe ordenar ascendente o descendentemente
     */
    public void ordenar(Ordenamiento ordenamiento, Comparador comparador, boolean ascendete)
    {
    	ordenador.ordenar(ordenamiento, cupiCava.darVinos(), ascendete, comparador.darComparator());
    	actualizarLista();
    }


    /**
     * Busca el vino por el nombre dado por el usuario.
     */
    public void buscarPorNombre( )
    {
        String nombre = JOptionPane.showInputDialog( null, "Nombre: ", "Buscar vino por nombre", JOptionPane.QUESTION_MESSAGE );
        if( nombre != null && !nombre.isEmpty( ) )
        {
            ordenador.ordenar(Ordenamiento.INSERCION, cupiCava.darVinos(), true, new ComparadorVinoNombre());
            actualizarLista( );
            Vino vino = cupiCava.buscarBinarioPorNombre( nombre );
            if( vino != null )
            {
                panelListaVinos.seleccionar( vino.darNombre( ) );
            }
            else
            {
                JOptionPane.showMessageDialog( this, "No se encontr ningn vino con el nombre dado.", "Buscar vino por nombre", JOptionPane.INFORMATION_MESSAGE );
            }
        }
    }

    /**
     * Busca los vinos con el tipo dado por el usuario.
     */
    public void buscarPorTipo( )
    {
        TipoVino tipo = ( TipoVino )JOptionPane.showInputDialog( null, "Tipo: ", "Buscar vinos de un tipo", JOptionPane.QUESTION_MESSAGE, null, TipoVino.values(), TipoVino.SECO );
        if( tipo != null )
        {

            List<Vino> vinosTipo = cupiCava.buscarVinosDeTipo( tipo );
                      
            panelListaVinos.refrescarLista( vinosTipo );
        }
    }

    /**
     * Busca el vino ms dulce.
     */
    public void buscarVinoMasDulce( )
    {
        Vino vino = cupiCava.buscarVinoMasDulce( );
        if( vino != null )
        {
            actualizarLista( );
            panelListaVinos.seleccionar( vino.darNombre( ) );
        }
        else
        {
            JOptionPane.showMessageDialog( this, "La cava no tiene vinos.", "Buscar vino ms dulce", JOptionPane.ERROR_MESSAGE );
        }
    }

    /**
     * Busca el vino ms seco.
     */
    public void buscarVinoMasSeco( )
    {
        Vino vino = cupiCava.buscarVinoMasSeco( );
        if( vino != null )
        {
            actualizarLista( );
            panelListaVinos.seleccionar( vino.darNombre( ) );
        }
        else
        {
            JOptionPane.showMessageDialog( this, "La cava no tiene vinos.", "Buscar vino ms seco", JOptionPane.ERROR_MESSAGE );
        }
    }

    /**
     * Agrega una nuevo vino a la cava con los valores dados.
     * @param pNombre Nombre del vino. pNombre != null && pNombre != "".
     * @param pPresentacion Presentacin del vino. pPresentacion != null && pPresentacion != "" && (pPresentacion == BOTELLA || pPresentacion == BARRIL).
     * @param pAnhoElaboracion Ao de elaboracin del vino. pAnhoElaboracion > 0.
     * @param pContenidoAzucar Contenido en azcar del vino. pContenidoAzucar >= 0
     * @param pColor Color del vino. pColor != null && pColor != "" && (pColor == TINTO || pColor == ROSADO || pColor == BLANCO).
     * @param pLugarOrigen Lugar de origen del vino. lugarElaboracion != null y lugarElaboracion != "".
     * @param pImagen Imagen del vino. pImagen != null && pImagen != "".
     */
    public void agregarVino( String pNombre, PresentacionVino pPresentacion, int pAnhoElaboracion, double pContenidoAzucar, ColorVino pColor, String pLugarOrigen, String pImagen )
    {
        boolean agregada = cupiCava.agregarVino( pNombre, pPresentacion, pAnhoElaboracion, pContenidoAzucar, pColor, pLugarOrigen, pImagen );
        if( agregada )
        {
            actualizarLista( );
        }
        else
        {
            JOptionPane.showMessageDialog( this, "Ya existe en la cava un vino con el nombre dado.", "Agregar vino", JOptionPane.ERROR_MESSAGE );
        }
    }

    // -----------------------------------------------------------------
    // Puntos de Extensin
    // -----------------------------------------------------------------

    /**
     * Mtodo para la extensin 1.
     */
    public void reqFuncOpcion1( )
    {
        String resultado = cupiCava.metodo1( );
        JOptionPane.showMessageDialog( this, resultado, "Respuesta", JOptionPane.INFORMATION_MESSAGE );
    }

    /**
     * Mtodo para la extensin 2.
     */
    public void reqFuncOpcion2( )
    {
        String resultado = cupiCava.metodo2( );
        JOptionPane.showMessageDialog( this, resultado, "Respuesta", JOptionPane.INFORMATION_MESSAGE );
    }

    // -----------------------------------------------------------------
    // Main
    // -----------------------------------------------------------------

    /**
     * Este mtodo ejecuta la aplicacin, creando una nueva interfaz.
     * @param pArgs Argumentos para la ejecucin de la aplicacin. En este caso no son necesarios. pArgs != null.
     */
    public static void main( String[] pArgs )
    {
        InterfazCupiCava interfaz = new InterfazCupiCava( );
        interfaz.setVisible( true );
    }
}