/**~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Licenciado bajo el esquema Academic Free License version 2.1 
 *
 * Proyecto Cupi2 (http://cupi2.uniandes.edu.co)
 * Ejercicio: n7_cupiCava
 * Autor: Equipo Cupi2 2015
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 */
package uniandes.cupi2.cupiCava.interfaz;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.TitledBorder;

import uniandes.cupi2.cupiCava.ordenamiento.Comparador;
import uniandes.cupi2.cupiCava.ordenamiento.Ordenamiento;

/**
 * Panel con las opciones de ordenamiento y bsqueda.
 */
public class PanelOpciones extends JPanel implements ActionListener
{
    // -----------------------------------------------------------------
    // Constantes
    // -----------------------------------------------------------------

    /**
     * Constante que representa el comando de ordenar.
     */
    private final static String ORDENAR = "Ordenar";

    /**
     * Constante que representa el comando de buscar.
     */
    private final static String BUSCAR = "Buscar";

      /**
     * Constante que representa la opcin de bsqueda por nombre.
     */
    private final static String BUSCAR_NOMBRE = "Nombre";

    /**
     * Constante que representa la opcin de bsqueda por tipo.
     */
    private final static String BUSCAR_TIPO = "Tipo";

    /**
     * Constante que representa la opcin de bsqueda del vino ms dulce.
     */
    private final static String BUSCAR_MAS_DULCE = "Ms dulce";

    /**
     * Constante que representa la opcin de bsqueda del vino ms seco.
     */
    private final static String BUSCAR_MAS_SECO = "Ms seco";

    // -----------------------------------------------------------------
    // Atributos
    // -----------------------------------------------------------------

    /**
     * Ventana principal de la aplicacin.
     */
    private InterfazCupiCava principal;

    // -----------------------------------------------------------------
    // Atributos de interfaz
    // -----------------------------------------------------------------

    /**
     * Combo box con los posibles cirterios de ordenamiento
     */
    private JComboBox comboAtributoOdenamiento;
    
    /**
     * Combo box con los posibles algoritmos de ordenamiento
     */
    //TODO 16.A Declare el atributo comboAlgoritmoOrdenamiento de tipo JComboBox para mostrar los algoritmos de ordenamiento
    
    /**
     * Campo de seleccin para indicar el sentido del ordenamiento
     */
    private JCheckBox cbAscendente;

    /**
     * Combo box con las opciones de bsqueda.
     */
    private JComboBox comboBusqueda;

    /**
     * Botn para ordenar.
     */
    private JButton btnOrdenar;

    /**
     * Botn para buscar.
     */
    private JButton btnBuscar;

    // -----------------------------------------------------------------
    // Constructores
    // -----------------------------------------------------------------

    /**
     * Constructor del panel.
     * @param pPrincipal Ventana principal de la aplicacin. pPrincipal != null.
     */
    public PanelOpciones( InterfazCupiCava pPrincipal )
    {
        principal = pPrincipal;

        setLayout( new GridLayout( 2, 1 ) );

        JPanel aux = new JPanel( );
        aux.setLayout( new GridLayout( 1, 4 ) );
        aux.setBorder( new CompoundBorder( new EmptyBorder( 0, 10, 0, 10 ), new TitledBorder( "Ordenar" ) ) );

        comboAtributoOdenamiento = new JComboBox();
        
        
        for (Comparador co : Comparador.values()) {
			comboAtributoOdenamiento.addItem(co);
		}
        
        aux.add( comboAtributoOdenamiento );
        
        //TODO 	16.B Inicialice el atributo comboAlgoritmoOrdenamiento 
        //		Recorra todos los elementos de la enumeracin Ordenaimento y agregue cada elemento al atributo comboAlgoritmoOrdenamiento
        //		Agregue el attributo comboAlgoritmoOrdenamiento al panel


        
        cbAscendente = new JCheckBox("Ascendente?");
        cbAscendente.setSelected(true);
        aux.add(cbAscendente);

        btnOrdenar = new JButton( ORDENAR );
        btnOrdenar.setActionCommand( ORDENAR );
        btnOrdenar.addActionListener( this );
        aux.add( btnOrdenar );

        add( aux );

        JPanel aux2 = new JPanel( );
        aux2.setLayout( new GridLayout( 1, 2 ) );
        aux2.setBorder( new CompoundBorder( new EmptyBorder( 0, 10, 0, 10 ), new TitledBorder( "Buscar" ) ) );

        comboBusqueda = new JComboBox( );
        comboBusqueda.setBorder( new EmptyBorder( 0, 0, 0, 10 ) );
        comboBusqueda.addItem( BUSCAR_NOMBRE );
        comboBusqueda.addItem( BUSCAR_TIPO );
        comboBusqueda.addItem( BUSCAR_MAS_DULCE );
        comboBusqueda.addItem( BUSCAR_MAS_SECO );
        aux2.add( comboBusqueda );

        btnBuscar = new JButton( BUSCAR );
        btnBuscar.setActionCommand( BUSCAR );
        btnBuscar.addActionListener( this );
        aux2.add( btnBuscar );

        add( aux2 );
    }

    /**
     * Manejo de los eventos de los botones.
     * @param pEvento Accin que gener el evento.
     */
    public void actionPerformed( ActionEvent pEvento )
    {
        String comando = pEvento.getActionCommand( );
        if( comando.equals( ORDENAR ) )
        {
        	Comparador c = (Comparador)comboAtributoOdenamiento.getSelectedItem();
        	//TODO 16.C Recupere el ordenamiento seleccionado en el atributo comboAlgoritmoOrdenamiento
        	
        	principal.ordenar(o, c, cbAscendente.isSelected());
           
        }
        else if( comando.equals( BUSCAR ) )
        {
            String busqueda = ( String )comboBusqueda.getSelectedItem( );
            if( busqueda.equals( BUSCAR_NOMBRE ) )
            {
                principal.buscarPorNombre( );
            }
            else if( busqueda.equals( BUSCAR_TIPO ) )
            {
                principal.buscarPorTipo( );
            }
            else if( busqueda.equals( BUSCAR_MAS_DULCE ) )
            {
                principal.buscarVinoMasDulce( );
            }
            else if( busqueda.equals( BUSCAR_MAS_SECO ) )
            {
                principal.buscarVinoMasSeco( );
            }
        }
    }
}