/**~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Licenciado bajo el esquema Academic Free License version 2.1 
 *
 * Proyecto Cupi2 (http://cupi2.uniandes.edu.co)
 * Ejercicio: n7_cupiCava
 * Autor: Equipo Cupi2 2015
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 */
package uniandes.cupi2.cupiCava.mundo;

import java.util.ArrayList;
import java.util.List;

/**
 * Clase que representa la Cupi-Cava. <br>
 * <b>inv: </b> <br>
 * //TODO 7.A Defina el invariante de la clase y documntelo.
 */
public class CupiCava
{
    // -------------------------------------------------------------
    // Atributos
    // -------------------------------------------------------------

    /**
     * Lista de vinos en la cava.
     */
    private List<Vino> vinos;

    // -------------------------------------------------------------
    // Mtodo Constructor
    // -------------------------------------------------------------

    /**
     * Construye una nueva cava sin vinos. <br>
     * <b>post:</b> La lista de vinos ha sido inicializada.
     */
    public CupiCava( )
    {
        vinos = new ArrayList<Vino>( );
    }

    // -------------------------------------------------------------
    // Mtodos
    // -------------------------------------------------------------

    /**
     * Retorna la lista de vinos.
     * @return Lista de vinos.
     */
    public List<Vino> darVinos( )
    {
        return vinos;
    }

    /**
     * Busca un vino con el nombre dado por parmetro. <br>
     * <b>pre:</b> La lista de vinos est inicializada.
     * @param pNombre Nombre del vino. pNombre != null && pNombre != ""
     * @return Vino con el nombre dado, null en caso de no encontrarlo.
     */
    public Vino buscarVino( String pNombre )
    {
        Vino buscado = null;
        boolean encontre = false;

        for( int i = 0; i < vinos.size( ) && !encontre; i++ )
        {
            Vino vinoActual = vinos.get( i );
            if( vinoActual.darNombre( ).equalsIgnoreCase( pNombre ) )
            {
                buscado = vinoActual;
                encontre = true;
            }
        }

        return buscado;
    }

    /**
     * Busca un vino utilizando una bsqueda binaria. <br>
     * <b>pre: </b> La lista de vinos est inicializada y se encuentra ordenada por nombre.
     * @param pNombre Nombre del vino que se va a buscar. pNombre != null && pNombre != "".
     * @return Vino con el nombre dado, null en caso de no encontrarlo.
     */
    public Vino buscarBinarioPorNombre( String pNombre )
    {
        //TODO 5.A Complete el mtodo segn la documentacin presentada
    }

    /**
     * Busca el vino ms dulce (con mayor contenido en azcar) de la cava. <br>
     * <b>pre:</b> La lista de vinos est inicializada.
     * @return Vino ms dulce de la cava. Si la cava no tiene vinos se retorna null. Si existen varios vinos con el contenido en azcar ms alto, se retorna el primer vino
     *         encontrado.
     */
    public Vino buscarVinoMasDulce( )
    {
    	//TODO 5.B Complete el mtodo segn la documentacin presentada
    }

    /**
     * Busca el vino ms seco (con menor contenido en azcar) de la cava. <br>
     * <b>pre:</b> La lista de vinos est inicializada.
     * @return Vino ms seco de la cava. Si la cava no tiene vinos se retorna null. Si existen varios vinos con el contenido en azcar ms bajo, se retorna el primer vino
     *         encontrado.
     */
    public Vino buscarVinoMasSeco( )
    {
    	//TODO 5.C Complete el mtodo segn la documentacin presentada
    	
    }

    /**
     * Busca los vinos del tipo dado por parmetro. <br>
     * <b>pre:</b> La lista de vinos est inicializada.
     * @param pTipo Tipo de vino de acuerdo a su contenido en azcar.pTipo != null && pTipo != "" && (pTipo == SECO || pTipo == ABOCADO || pTipo == SEMI_SECO || pTipo ==
     *        SEMI_DULCE || pTipo == DULCE).
     * @return Lista de vinos del tipo dado.
     */
    public List<Vino> buscarVinosDeTipo( TipoVino pTipo )
    {
    	//TODO 5.C Complete el mtodo segn la documentacin presentada
    }

    /**
     * Agrega un nuevo vino a la cava si no existe actualmente un vino en la cava con el mismo nombre.<br>
     * <b>pre:</b> La lista de vinos est inicializada.<br>
     * <b>post:</b> Se agreg un nuevo vino a la lista de vinos.<br>
     * @param pNombre Nombre del vino. pNombre != null && pNombre != "".
     * @param pPresentacion Presentacin del vino. pPresentacion != null && pPresentacion != "" && (pPresentacion == BOTELLA || pPresentacion == BARRIL).
     * @param pAnhoElaboracion Ao de elaboracin del vino. pAnhoElaboracion > 0.
     * @param pContenidoAzucar Contenido en azcar del vino. pContenidoAzucar >= 0
     * @param pTipo Tipo de vino de acuerdo a su contenido en azcar. pTipo != null && pTipo != "" && (pTipo == SECO || pTipo == ABOCADO || pTipo == SEMI_SECO || pTipo ==
     *        SEMI_DULCE || pTipo == DULCE).
     * @param pColor Color del vino. pColor != null && pColor != "" && (pColor == TINTO || pColor == ROSADO || pColor == BLANCO).
     * @param pLugarOrigen Lugar de origen del vino. lugarElaboracion != null y lugarElaboracion != "".
     * @param pImagen Imagen del vino. pImagen != null && pImagen != "".
     * @return True si el vino es agregado, false de lo contrario.
     */
    public boolean agregarVino( String pNombre, PresentacionVino pPresentacion, int pAnhoElaboracion, double pContenidoAzucar, ColorVino pColor, String pLugarOrigen, String pImagen )
    {
        Vino buscado = buscarVino( pNombre );
        boolean agregada = false;

        if( buscado == null )
        {
            Vino vino = new Vino( pNombre, pPresentacion, pAnhoElaboracion, pContenidoAzucar, pColor, pLugarOrigen, pImagen );
            vinos.add( vino );
            agregada = true;
        }
        return agregada;
    }

    // -----------------------------------------------------------------
    // Invariante
    // -----------------------------------------------------------------

    // TODO 7.B Cree e implemente el mtodo private void verificarInvariante para validar el invariante definido en Parte1 PuntoC. De ser necesario puede crear
    // mtodos de soporte
    
    
    // -----------------------------------------------------------------
    // Puntos de Extensin
    // -----------------------------------------------------------------


	/**
     * Mtodo para la extensin 1.
     * @return Respuesta 1.
     */
    public String metodo1( )
    {
        return "Respuesta 1.";
    }

    /**
     * Mtodo para la extensin 2.
     * @return Respuesta 2.
     */
    public String metodo2( )
    {
        return "Respuesta 2.";
    }
}