package uniandes.cupi2.cupiCava.ordenamiento;

import java.util.Comparator;

import uniandes.cupi2.cupiCava.mundo.Vino;

/**
 * Enumeracin de comparadores que existen sobre los vinos
 * @author alvar-go
 */
public enum Comparador {
	ANHO ("Ao de elaboracin", new ComparadorVinoAnho()),
	LUGAR ("Lugar de origen", new ComparadorVinoLugar()),
	NOMBRE ("Nombre", new ComparadorVinoNombre());
	
	/**
	 * Nombre del atributo por el que se realiza la comparacin
	 */
	private final String nombre;
	
	/**
	 * Clase responsable de hacer la comparacin
	 */
	private final Comparator<Vino> comparator;
	
	/**
	 * Crea un nuevo elemento de la enumeracin
	 * @param nNombre el nombre del atributo por le que se realiza la coparacin
	 * @param nComparator clase que realiza la comparacin
	 */
	private Comparador(String nNombre, Comparator<Vino> nComparator)
	{
		nombre = nNombre;
		comparator = nComparator;
				
	}
	
	/**
	 * Devuelve el nombre del atributo por el que se puede hacer la comparacin
	 * @return nombre
	 */
	public String darNombre()
	{
		return nombre;
	}
	
	/**
	 * Devuelve la clase que realiza la comparacin 
	 * @return comparator
	 */
	public Comparator<Vino> darComparator()
	{
		return comparator;
	}
	
	/**
	 * Representacin en String del comparador
	 * @return nombre
	 */
	public String toString()
	{
		return nombre;
	}
}
