package uniandes.cupi2.cupiCava.ordenamiento;

import java.util.Comparator;
import java.util.List;

/**
 * Clase gnerica que se encarga de ordenar colecciones de elementos de tipo T
 * @author alvar-go
 * @param <T> tipo de elementos que debe contener la coleccin a ordenar
 */
public class Ordenador<T> 
{
	/**
	 * Ordena la lista que llega por parmetro en el sentido que llega por parmetro usando el comparador y el ordenamiento que llegan por parmetro
	 * post: la lista se encuentra ordenada segn los cirterios ingresados
	 * @param ordenamiento el algoritmo de ordenamieto que se debe usar. ordenamiento pertenece a la enumeracin Ordenamiento
	 * @param lista la lista que se desea ordenar. lista != null
	 * @param ascendente indica si se debe ordenar de manera ascendente, en caso que sea false se debe ordenar descendentemente
	 * @param comparador comparator de elementos tipo T que se usar para ordenar la lista, define el criterio de orden. comparador != null.
	 */
	public void ordenar(Ordenamiento ordenamiento, List<T> lista, boolean ascendente, Comparator<T> comparador)
	{
		if(ordenamiento.equals(Ordenamiento.BURBUJA))
		{
			ordenarBurbuja(lista, ascendente, comparador);
		}
		else if(ordenamiento.equals(Ordenamiento.SELECCION))
		{
			ordenarSeleccion(lista, ascendente, comparador);
		}
		else if(ordenamiento.equals(Ordenamiento.INSERCION))
		{
			ordenarInsercion(lista, ascendente, comparador);
		}
		else if(ordenamiento.equals(Ordenamiento.SHAKER))
		{
			ordenarShaker(lista, ascendente, comparador);
		}
		else if(ordenamiento.equals(Ordenamiento.GNOME))
		{
			ordenarGnome(lista, ascendente, comparador);
		}
	}

	/**
	 * Ordena la lista usando el algoritmo de inscercin
	 * post: la lista se encuentra ordenada
	 * @param lista la lista que se desea ordenar. lista != null
	 * @param ascendnte indica si se debe ordenar de mamenra ascendente, en caso que sea false se ordenar de manera descendente
	 * @param comparator comparador de elementos tipo T que se usar para ordenar la lista, define el criterio de orden. comparador != null.
	 */
	private void ordenarInsercion(List<T> lista,boolean ascendnte, Comparator<T> comparador) {
		// TODO 14.A Ordena la lista usando el algoritmo de burbuja. Recuerde
		//		1. Revise el sentido en que se debe ordenar la lista (ascendente / descendente)
		//		2. Use el comparador como criterio de ordenamiento
	}

	/**
	 * Ordena la lista usando el algoritmo de seleccin
	 * post: la lista se encuentra ordenada
	 * @param lista la lista que se desea ordenar. lista != null
	 * @param ascendnte indica si se debe ordenar de mamenra ascendente, en caso que sea false se ordenar de manera descendente
	 * @param comparator comparador de elementos tipo T que se usar para ordenar la lista, define el criterio de orden. comparador != null.
	 */
	private void ordenarSeleccion(List<T> lista,boolean ascendnte, Comparator<T> comparador) {
		// TODO 14.B Ordena la lista usando el algoritmo de seleccin. Recuerde
				//		1. Revise el sentido en que se debe ordenar la lista (ascendente / descendente)
				//		2. Use el comparador como criterio de ordenamiento
		
	}

	/**
	 * Ordena la lista usando el algoritmo de burbuja
	 * post: la lista se encuentra ordenada
	 * @param lista la lista que se desea ordenar. lsita != null
	 * @param ascendnte indica si se debe ordenar de mamenra ascendente, de lo contrario se ordenar de manera descendente
	 * @param comparador comparador de elementos tipo T que se usar para ordenar la lista, define el criterio de orden. comparador != null.
	 */
	private void ordenarBurbuja(List<T> lista, boolean ascendnte, Comparator<T> comparador) {
		// TODO 14.C Ordena la lista usando el algoritmo de burbuja. Recuerde
		//		1. Revise el sentido en que se debe ordenar la lista (ascendente / descendente)
		//		2. Use el comparador como criterio de ordenamiento
	}
	
	/**
	 * Ordena la lista usando el algoritmo de shake (burbuja bidireccional)
	 * post: la lista se encuentra ordenada
	 * @param lista la lista que se desea ordenar. lista != null
	 * @param ascendnte indica si se debe ordenar de mamenra ascendente, en caso que sea false se ordenar de manera descendente
	 * @param comparator comparador de elementos tipo T que se usar para ordenar la lista, define el criterio de orden. comparador != null.
	 */
	private void ordenarShaker(List<T> lista,boolean ascendnte, Comparator<T> comparador)
	{
		// TODO 14.D Ordena la lista usando el algoritmo de shake. Recuerde
		//		1. Revise el sentido en que se debe ordenar la lista (ascendente / descendente)
		//		2. Use el comparador como criterio de ordenamiento
	}
		
	/**
	 * Ordena la lista usando el algoritmo de Gnome
	 * post: la lista se encuentra ordenada
	 * @param lista la lista que se desea ordenar. lista != null
	 * @param ascendnte indica si se debe ordenar de mamenra ascendente, en caso que sea false se ordenar de manera descendente
	 * @param comparator comparador de elementos tipo T que se usar para ordenar la lista, define el criterio de orden. comparador != null.
	 */
	private void ordenarGnome(List<T> lista,boolean ascendnte, Comparator<T> comparador) {
		
		// TODO 14.E Ordena la lista usando el algoritmo Gnome. Recuerde
		//		1. Revise el sentido en que se debe ordenar la lista (ascendente / descendente)
		//		2. Use el comparador como criterio de ordenamiento
	}
}
