/**~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Licenciado bajo el esquema Academic Free License version 2.1 
 *
 * Proyecto Cupi2 (http://cupi2.uniandes.edu.co)
 * Ejercicio: n7_cupiCava
 * Autor: Equipo Cupi2 2015
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 */
package uniandes.cupi2.cupiCava.test;

import java.util.List;

import uniandes.cupi2.cupiCava.mundo.ColorVino;
import uniandes.cupi2.cupiCava.mundo.CupiCava;
import uniandes.cupi2.cupiCava.mundo.PresentacionVino;
import uniandes.cupi2.cupiCava.mundo.TipoVino;
import uniandes.cupi2.cupiCava.mundo.Vino;
import uniandes.cupi2.cupiCava.ordenamiento.ComparadorVinoNombre;
import uniandes.cupi2.cupiCava.ordenamiento.Ordenador;
import uniandes.cupi2.cupiCava.ordenamiento.Ordenamiento;
import junit.framework.TestCase;

/**
 * Clase usada para verificar que los mtodos de la clase Cava estn correctamente implementados.
 */
public class CupiCavaTest extends TestCase
{
    // -------------------------------------------------------------
    // Atributos
    // -------------------------------------------------------------

    /**
     * Clase donde se harn las pruebas.
     */
    private CupiCava cupiCava1;
    
    /**
     * Clase donde se harn las pruebas.
     */
    private CupiCava cupiCava2;
    
    /**
     * Clase donde se harn las pruebas.
     */
    private CupiCava cupiCava3;

    // -------------------------------------------------------------
    // Mtodos
    // -------------------------------------------------------------

    /**
     * Inicializa cada una de las cavas para hacer las pruebas
     */
    protected void setUp()
    {
    	cupiCava1 = new CupiCava( );
    	
    	 cupiCava2 = new CupiCava( );

         cupiCava2.agregarVino( "Alamos Chardonnay", PresentacionVino.BOTELLA, 2006, 35, ColorVino.BLANCO, "Mendoza, Argentina", "imagen1" );
         cupiCava2.agregarVino( "Marqus de Grion Reserva", PresentacionVino.BARRIL, 2000, 7,  ColorVino.TINTO, "Toledo, Espaa", "imagen2" );
         cupiCava2.agregarVino( "Castillo de Javier", PresentacionVino.BOTELLA, 2006, 20,  ColorVino.ROSADO, "Navarra, Espaa", "imagen3" );
         
         cupiCava3 = new CupiCava( );

         cupiCava3.agregarVino( "Alamos Chardonnay", PresentacionVino.BOTELLA, 2006, 35,  ColorVino.BLANCO, "Mendoza, Argentina", "imagen1" );
         cupiCava3.agregarVino( "Mont-Ferrant Brut Nature Vintage", PresentacionVino.BARRIL, 2010, 2,  ColorVino.BLANCO, "Espaa", "imagen2" );
         cupiCava3.agregarVino( "Marqus de Grion Reserva", PresentacionVino.BARRIL, 2000, 7, ColorVino.TINTO, "Toledo, Espaa", "imagen3" );
         cupiCava3.agregarVino( "Camino Real", PresentacionVino.BOTELLA, 2008, 15,  ColorVino.TINTO, "Chile", "imagen4" );
         cupiCava3.agregarVino( "Conde de Rioja", PresentacionVino.BOTELLA, 2010, 25,  ColorVino.BLANCO, "Colombia", "imagen5" );
         cupiCava3.agregarVino( "Castillo de Javier", PresentacionVino.BOTELLA, 2006, 20,  ColorVino.ROSADO, "Navarra, Espaa", "imagen6" );
         cupiCava3.agregarVino( "Tavel Aoc", PresentacionVino.BOTELLA, 2008, 55, ColorVino.ROSADO, "Francia", "imagen7" );
    }
    

    /**
     * Prueba 1: Se encarga de verificar el mtodo constructor de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * darVinos<br>
     * Cava<br>
     * <b> Caso de prueba 1:</b><br>
     * Construye correctamente la cava, cada uno de los valores corresponde al esperado.<br>
     */
    public void testCava( )
    {

        assertNotNull( "La lista de vinos no puede ser nula.", cupiCava1.darVinos( ) );
        assertEquals( "La lista de vinos debi crearse vaca.", 0, cupiCava1.darVinos( ).size( ) );
    }

    /**
     * Prueba 2: Se encarga de verificar el mtodo agregarVino de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * agregarVino<br>
     * darVinos<br>
     * <b> Caso de prueba 1:</b><br>
     * No hay vinos en la cava.<br>
     */
    public void testAgregarVino1( )
    {


        // Verificar que se haya agregado el vino
        boolean vinoAgregado = cupiCava1.agregarVino( "Alamos Chardonnay", PresentacionVino.BOTELLA, 2006, 35, ColorVino.BLANCO, "Mendoza, Argentina", "imagen1" );
        assertTrue( "Siempre se debe poder agregar el primer vino.", vinoAgregado );

        List<Vino> vinos = cupiCava1.darVinos( );
        assertEquals( "El vino no se agreg a la lista de vinos.", 1, vinos.size( ) );

        // Datos del vino que se agreg en la lista
        Vino vino = vinos.get( 0 );
        assertEquals( "El nombre del vino no es el esperado.", "Alamos Chardonnay", vino.darNombre( ) );
        assertEquals( "La presentacin del vino no es la esperada.", PresentacionVino.BOTELLA, vino.darPresentacion( ) );
        assertEquals( "El ao de elaboracin del vino no es el esperado.", 2006, vino.darAnhoElaboracion( ) );
        assertEquals( "El contenido en azcar del vino no es el esperado.", 35.0, vino.darContenidoAzucar( ) );
        assertEquals( "El tipo del vino no es el esperado.", TipoVino.SEMI_DULCE, vino.darTipo( ) );
        assertEquals( "El color del vino no es el esperado.", ColorVino.BLANCO, vino.darColor( ) );
        assertEquals( "El lugar de origen del vino no es el esperado.", "Mendoza, Argentina", vino.darLugarOrigen( ) );
    }

    /**
     * Prueba 3: Se encarga de verificar el mtodo agregarVino de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * agregarVino<br>
     * darVinos<br>
     * <b> Caso de prueba 1:</b><br>
     * Hay vinos en la cava.<br>
     */
    public void testAgregarVino2( )
    {


        // Verificar que se haya ingresado el vino
        boolean vinoAgregado = cupiCava2.agregarVino( "Mont-Ferrant", PresentacionVino.BARRIL, 2010, 2, ColorVino.BLANCO, "Espaa", "imagen4" );
        assertTrue( "Error al agregar el vino", vinoAgregado );

        List<Vino> vinos = cupiCava2.darVinos( );
        assertEquals( "El vino no se agreg a la lista de vinos", 4, vinos.size( ) );

        // Datos del vino que se agreg en la lista
        Vino vino = vinos.get( 3 );
        assertEquals( "El nombre del vino no es el esperado.", "Mont-Ferrant", vino.darNombre( ) );
        assertEquals( "La presentacin del vino no es la esperada.", PresentacionVino.BARRIL, vino.darPresentacion( ) );
        assertEquals( "El ao de elaboracin del vino no es el esperado.", 2010, vino.darAnhoElaboracion( ) );
        assertEquals( "El contenido en azcar del vino no es el esperado.", 2.0, vino.darContenidoAzucar( ) );
        assertEquals( "El tipo del vino no es el esperado.", TipoVino.SECO, vino.darTipo( ) );
        assertEquals( "El color del vino no es el esperado.", ColorVino.BLANCO, vino.darColor( ) );
        assertEquals( "El lugar de origen del vino no es el esperado.", "Espaa", vino.darLugarOrigen( ) );
    }

    /**
     * Prueba 4: Se encarga de verificar el mtodo agregarVino de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * agregarVino<br>
     * darVinos<br>
     * <b> Caso de prueba 1:</b><br>
     * Ya existe un vino en la cava con el nombre del que se quiere agregar.<br>
     */
    public void testAgregarVinoError( )
    {


        // Verificar que no se haya agregado el vino
        boolean vinoAgregado = cupiCava2.agregarVino( "Alamos Chardonnay", PresentacionVino.BOTELLA, 2006, 35, ColorVino.BLANCO, "Mendoza, Argentina", "nombre" );
        assertFalse( "No debi haber agregado el vino.", vinoAgregado );
        assertEquals( "El nmero de vinos de la cava no debi cambiar.", 3, cupiCava2.darVinos( ).size( ) );
    }

    /**
     * Prueba 5: Se encarga de verificar el mtodo buscarVino de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * buscarVino<br>
     * <b> Casos de prueba:</b><br>
     * 1. El vino existe.<br>
     * 2. El vino no existe.
     */
    public void testBuscarVino( )
    {


        Vino buscado = cupiCava2.buscarVino( "Castillo de Javier" );
        assertNotNull( "Debi haber encontrado un vino con el nombre dado.", buscado );

        assertNull( "No debi haber encontrado un vino con el nombre dado.", cupiCava2.buscarVino( "inexistente" ) );
    }
   

    /**
     * Prueba 6: Se encarga de verificar el mtodo buscarBinarioPorNombre de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * ordenarVinosPorNombre<br>
     * buscarBinarioPorNombre<br>
     * darNombre<br>
     * <b> Casos de prueba:</b><br>
     * 1. El vino existe.<br>
     * 2. El vino no existe.<br>
     */
    public void testBuscarBinarioPorNombre( )
    {
    	Ordenador<Vino> o = new Ordenador<Vino>();
    	o.ordenar(Ordenamiento.INSERCION, cupiCava3.darVinos(), true, new ComparadorVinoNombre());
    	
    	//TODO 	4.A Pruebe que el mtodo buscarBinarioPorNombre funciona correctamente.
    	//		Pruebe los dos casos presentado en la documentacin
    }

    /**
     * Prueba 7: Se encarga de verificar el mtodo buscarVinoMasDulce de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * buscarVinoMasDulce<br>
     * darVinos<br>
     * darNombre<br>
     * <b> Casos de prueba:</b><br>
     * 1. El vino ms dulce existe.<br>
     */
    public void testBuscarVinoMasDulce( )
    {
    	//TODO 	4.B Pruebe que el mtodo buscarVinoMasDulce funciona correctamente.
    	//		Pruebe el caso presentado en la documentacin
    }

    /**
     * Prueba 8: Se encarga de verificar el mtodo buscarVinoMasSeco de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * buscarVinoMasSeco<br>
     * darVinos<br>
     * darNombre<br>
     * <b> Casos de prueba:</b><br>
     * 1. El vino ms seco existe.<br>
     */
    public void testBuscarVinoMasSeco( )
    {
    	//TODO 	4.C Pruebe que el mtodo buscarVinoMasSeco funciona correctamente.
    	//		Pruebe el caso presentado en la documentacin
    }

    /**
     * Prueba 9: Se encarga de verificar el mtodo buscarVinosDeTipo de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * buscarVinosDeTipo<br>
     * darTipo<br>
     * <b> Casos de prueba:</b><br>
     * 1. Existen vinos con el tipo dado.<br>
     */
    public void testBuscarVinosDeTipo( )
    {


        List<Vino> vinos = cupiCava3.buscarVinosDeTipo( TipoVino.DULCE );
        assertEquals( "No se realiz la bsqueda correctamente.", 1, vinos.size( ) );

        for( int i = 0; i < vinos.size( ); i++ )
        {
            Vino vino = ( Vino )vinos.get( i );
            assertEquals( "No se buscaron correctamente los vinos.", vino.darTipo( ), TipoVino.DULCE  );
        }

        vinos = cupiCava3.buscarVinosDeTipo( TipoVino.SEMI_SECO  );
        assertEquals( "No se realiz la bsqueda correctamente.", 3, vinos.size( ) );

        for( int i = 0; i < vinos.size( ); i++ )
        {
            Vino vino = ( Vino )vinos.get( i );
            assertEquals( "No se buscaron correctamente los vinos.", vino.darTipo( ), TipoVino.SEMI_SECO );
        }
    }
}