package uniandes.cupi2.cupiCava.test;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import uniandes.cupi2.cupiCava.mundo.ColorVino;
import uniandes.cupi2.cupiCava.mundo.PresentacionVino;
import uniandes.cupi2.cupiCava.mundo.Vino;
import uniandes.cupi2.cupiCava.ordenamiento.Comparador;
import uniandes.cupi2.cupiCava.ordenamiento.Ordenador;
import uniandes.cupi2.cupiCava.ordenamiento.Ordenamiento;
import junit.framework.TestCase;

/**
 * Clase encarada de probar la clase que realiza ordenamientos
 * @author alvar-go
 *
 */
public class OrdenadorTest extends TestCase {
	
	 // -------------------------------------------------------------
    // Atributos
    // -------------------------------------------------------------

    /**
     * Lsita que se usara para realizar las pruebas
     */
    private List<Vino> vinos;
    
    /**
     * Clase que se desea probar
     */
    private Ordenador<Vino> ordenador;
    

    // -------------------------------------------------------------
    // Mtodos
    // -------------------------------------------------------------

    /**
     * Inicializa la lista de vinos para probar los ordenamientos
     * Inicializa el ordenador
     */
    protected void setUp()
    {

         vinos = new ArrayList<Vino>();

         vinos.add(new Vino ("Alamos Chardonnay", PresentacionVino.BOTELLA, 2006, 35,  ColorVino.BLANCO, "Mendoza, Argentina", "imagen1" ));
         vinos.add(new Vino ( "Mont-Ferrant Brut Nature Vintage", PresentacionVino.BARRIL, 2010, 2,  ColorVino.BLANCO, "Espaa", "imagen2" ));
         vinos.add(new Vino ( "Marqus de Grion Reserva", PresentacionVino.BARRIL, 2000, 7, ColorVino.TINTO, "Toledo, Espaa", "imagen3" ));
         vinos.add(new Vino ( "Camino Real", PresentacionVino.BOTELLA, 2008, 15,  ColorVino.TINTO, "Chile", "imagen4" ));
         vinos.add(new Vino ( "Conde de Rioja", PresentacionVino.BOTELLA, 2010, 25,  ColorVino.BLANCO, "Colombia", "imagen5" ));
         vinos.add(new Vino ( "Castillo de Javier", PresentacionVino.BOTELLA, 2006, 20,  ColorVino.ROSADO, "Navarra, Espaa", "imagen6" ));
         vinos.add(new Vino ( "Tavel Aoc", PresentacionVino.BOTELLA, 2008, 55, ColorVino.ROSADO, "Francia", "imagen7" ));
         
         ordenador = new Ordenador<Vino>();
    }

	 /**
     * Prueba 1: Se encarga de verificar todos los algoritmos de ordenamiento por todos los criterios de forma ascendente.<br>
     * <b> Mtodos a probar: </b> <br>
     * ordenar<br>
     * <b> Casos de prueba:</b><br>
     * 1. Los vinos no estn ordenados.<br>
     */
    public void testOrdenarAscendente( )
    {
    	
    	for (Ordenamiento or : Ordenamiento.values()) {
			for (Comparador co : Comparador.values()) {
				
				int tamPrevio = vinos.size();
				Collections.shuffle(vinos);
				ordenador.ordenar(or, vinos, true, co.darComparator());
				
									
				for (int i = 0; i < vinos.size()-1; i++) {
					Vino v1 = vinos.get(i);
					Vino v2 = vinos.get(i+1);
					assertTrue("Ordenamiento ascendente por " + co.darNombre() + " y " + or.darNombre() + " es incorrecto", co.darComparator().compare(v1, v2) <= 0);
				}
				assertEquals("Se borraron elementos en " + or.darNombre(), tamPrevio, vinos.size());
				
			}
		}

    }
    
	 /**
     * Prueba 1: Se encarga de verificar todos los algoritmos de ordenamiento por todos los criterios de forma descendente.<br>
     * <b> Mtodos a probar: </b> <br>
     * ordenar<br>
     * <b> Casos de prueba:</b><br>
     * 1. Los vinos no estn ordenados.<br>
     */
    public void testOrdenarDescendente( )
    {
    	//Itera sobre todos los ordenamientos registrados
    	for (Ordenamiento or : Ordenamiento.values()) {
    		//Itera sobre todos los comparadores registrados
			for (Comparador co : Comparador.values()) {
				//Desordena la lista de vinos
				Collections.shuffle(vinos);
				
				//TODO 11.A Verifica que el ordenador ordena correctamente usando los siguientes parametros
				//			Ordenador = or
				//			Comparador = co.darComparator
				//			Ascendnete = false
			}
		}

    }

   
}
